<?php

/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | jh_php_pdf version 1.1.0                                             |
// +----------------------------------------------------------------------+
// | Copyright (c) 2002 Johann Hanne                                      |
// +----------------------------------------------------------------------+
// | This is free software; you can redistribute it and/or                |
// | modify it under the terms of the GNU Lesser General Public           |
// | License as published by the Free Software Foundation; either         |
// | version 2.1 of the License, or (at your option) any later version.   |
// |                                                                      |
// | This software is distributed in the hope that it will be useful,     |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
// | Lesser General Public License for more details.                      |
// |                                                                      |
// | You should have received a copy of the GNU Lesser General Public     |
// | License along with this software; if not, write to the               |
// | Free Software Foundation, Inc., 59 Temple Place,                     |
// | Suite 330, Boston, MA  02111-1307 USA                                |
// +----------------------------------------------------------------------+
// | Author: Johann Hanne <jonny@1409.org>                                |
// +----------------------------------------------------------------------+

require_once "class.jh_pdf_base-1.1.0.inc.php";

/**
* Class for barcode rendering
*
* Renders barcodes directly onto a PDF document using lines
* with different widths.
*
* @access   public
*/
class jh_pdf_barcode extends jh_pdf_base
{
    var $pdf;
    var $text;
    var $type;

    /**
    * Constructor
    *
    * @param    resource   &$pdf   Handle for the PDF document where
    *                              everything should be put onto
    *
    * @access   public
    */
    function jh_pdf_barcode(&$pdf) {
        $this->pdf=&$pdf;
        $this->type="39";
    }

    /**
    * Set the text
    *
    * Sets the text which should later be output as barcode.
    *
    * @param    string   $text   The string to output as barcode
    *
    * @access   public
    */
    function settext($text) {
        $this->text=$text;
    }

    /**
    * Set the type
    *
    * Sets the barcode type that will be output.
    *
    * @param    string   $type   The barcode type
    *
    * @access   public
    */
    function settype($type) {
        $this->type=strtolower($type);
    }

    /**
    * Output the barcode
    *
    * Puts the barcode specified via settext() onto the PDF document.
    * The barcode is scaled to fit into the specified space.
    *
    * @param    float    $xpos     The left starting position
    * @param    float    $ypos     The top starting position
    * @param    float    $width    The width of the barcode
    * @param    float    $height   The height of the barcode
    *
    * @access   public
    */
    function putcode($xpos, $ypos, $width, $height) {
        switch($this->type) {

        default:
        case '39':
            // code 39: narrow=4, wide=11, quiet=7
            $narrow=4;
            $wide=11;
            $quiet=7;

            $convtable[0]=$narrow;
            $convtable[1]=$wide;
            $convtable[2]=$quiet;

            $text='*'.$this->text.'*';
            $codes=array();
            for ($i=0;$i<strlen($text);$i++) {
                array_push($codes, $this->char2code39($text[$i]));
            }
            $code=implode('2', $codes);

            // Total width is the sum of:
            // - Widths of the characters' barcode representation (plus
            //   start/stop characters)
            // - Widths of the blank spaces between them
            $fullwidth=strlen($text)*((6*$narrow)+(3*$wide))+
                       (strlen($text)-1)*$quiet;
            $basewidth=$width/$fullwidth;
            break;

        case '25i':
            // code 25i: FIXME!!!
            $narrow=1;
            $wide=2;

            $convtable[0]=$narrow;
            $convtable[1]=$wide;

            // The length of the text must be even; if not, we
            // pad it with a leading "0"
            $text=$this->text;
            if (strlen($text)%2!=0) {
              $text="0".$text;
            }

            $code=$this->char2code25i('START');;
            for ($i=0;$i<strlen($text);$i+=2) {
                $code1=$this->char2code25i($text[$i]);
                $code2=$this->char2code25i($text[$i+1]);
                // Interleaved code: Shake 'em up!
                for($j=0;$j<5;$j++) {
                  $code.=$code1[$j];
                  $code.=$code2[$j];
                }
            }
            $code.=$this->char2code25i('STOP');;

            // Total width is the sum of:
            // - Widths of the start/stop characters (7*narrow+1*wide)
            // - Widths of the characters' barcode representation
            $fullwidth=strlen($text)*((3*$narrow)+(2*$wide))+
                       7*$narrow+1*$wide;
            $basewidth=$width/$fullwidth;
            break;
        }

        $black=true;
        for ($i=0;$i<strlen($code);$i++) {
            $barwidth=$convtable[$code[$i]]*$basewidth;

            if ($black) {
                PDF_setcolor($this->pdf, "both", "gray", 0, 0, 0, 0);
                PDF_setlinecap($this->pdf, 0);
                PDF_setlinewidth($this->pdf, $barwidth);
                PDF_moveto($this->pdf, $xpos+0.5*$barwidth, $ypos);
                PDF_lineto($this->pdf, $xpos+0.5*$barwidth, $ypos-$height);
                PDF_stroke($this->pdf);
            }

            $xpos+=$barwidth;

            $black=!$black;
        }

        return 1;
    }

    /**
    * Convert a character into its code 39 barcode representation
    *
    * Converts a character into a string consisting of 0s and 1s
    * representing its code 39 representation in narrow (0) and wide (1)
    * bars.
    *
    * @param    string    The character to be converted
    *
    * @access   private
    */        
    function char2code39($char) {
        switch ($char) {
        case ' ':
        default:
            return '011000100';
        case '$':
            return '010101000';
        case '%':
            return '000101010';
        case 'START':
        case 'STOP':
        case '*':
            return '010010100';
        case '+':
            return '010001010';
        case '|':
            return '010000101';
        case '.':
            return '110000100';
        case '/':
            return '010100010';
        case '0':
            return '000110100';
        case '1':
            return '100100001';
        case '2':
            return '001100001';
        case '3':
            return '101100000';
        case '4':
            return '000110001';
        case '5':
            return '100110000';
        case '6':
            return '001110000';
        case '7':
            return '000100101';
        case '8':
            return '100100100';
        case '9':
            return '001100100';
        case 'A':
            return '100001001';
        case 'B':
            return '001001001';
        case 'C':
            return '101001000';
        case 'D':
            return '000011001';
        case 'E':
            return '100011000';
        case 'F':
            return '001011000';
        case 'G':
            return '000001101';
        case 'H':
            return '100001100';
        case 'I':
            return '001001100';
        case 'J':
            return '000011100';
        case 'K':
            return '100000011';
        case 'L':
            return '001000011';
        case 'M':
            return '101000010';
        case 'N':
            return '000010011';
        case 'O':
            return '100010010';
        case 'P':
            return '001010010';
        case 'Q':
            return '000000111';
        case 'R':
            return '100000110';
        case 'S':
            return '001000110';
        case 'T':
            return '000010110';
        case 'U':
            return '110000001';
        case 'V':
            return '011000001';
        case 'W':
            return '111000000';
        case 'X':
            return '010010001';
        case 'Y':
            return '110010000';
        case 'Z':
            return '011010000';
        }
    }

    /**
    * Convert a character into its code 25 interleaved barcode representation
    *
    * Converts a character into a string consisting of 0s and 1s
    * representing its code 25 interleaved representation in narrow (0) and
    * wide (1)
    * bars.
    *
    * @param    string    The character to be converted
    *
    * @access   private
    */
    function char2code25i($char) {
        switch ($char) {
        case '0':
            return '00110';
        case '1':
            return '10001';
        case '2':
            return '01001';
        case '3':
            return '11000';
        case '4':
            return '00101';
        case '5':
            return '10100';
        case '6':
            return '01100';
        case '7':
            return '00011';
        case '8':
            return '10010';
        case '9':
            return '01010';
        case 'START':
            return '0000';
        case 'STOP':
            return '1000';
        default:
            return '';
        }
    }

}

?>
